@@ -50,11 +50,13 @@ def to_alfa_labels(labels, alfa_labels):
 def to_alfa_label(label, alfa_labels):
     """Convert YOLO to ALFA class ids/label."""
     if label in dataset_classnames['YOLO CAR']:  # YOLO car
-        alfa_labels.append(5)  # ALFA car 6 - 1 = 5, null-base-index
+        alfa_labels.append(dataset_classnames['PASCAL VOC'].index('car'))
     elif label in dataset_classnames['YOLO PERSON']:  # YOLO pedestrian
-        alfa_labels.append(13)  # ALFA person 14 - 1 = 13, null-base-index
+        alfa_labels.append(dataset_classnames['PASCAL VOC'].index(
+            'person'))  # ALFA person
     elif label in dataset_classnames['YOLO BICYCLE']:  # YOLO cyclist
-        alfa_labels.append(1)  # ALFA bicycle 2 - 1 = 1, null-base-index
+        alfa_labels.append(dataset_classnames['PASCAL VOC'].index(
+            'bicycle'))  # ALFA bicycle
 
 
 def to_ai4adas_label(label, ai4adas_labels):
@@ -65,11 +67,14 @@ def to_ai4adas_label(label, ai4adas_labels):
 
 def convert_label_id_to_ai4adas_label(label_id):
     """Convert ALFA class id to YOLO label."""
-    if label_id == 5:  # ALFA car 6 - 1 = 5, null-base-index
+    if label_id == dataset_classnames['PASCAL VOC'].index(
+            'car'):
         return "car"  # YOLO Car
-    elif label_id == 13:  # ALFA person 14 - 1 = 13, null-base-index
+    elif label_id == dataset_classnames['PASCAL VOC'].index(
+            'person'):
         return "pedestrian"  # YOLO Pedestrian
-    elif label_id == 1:  # ALFA bicycle 2 - 1 = 1, null-base-index
+    elif label_id == dataset_classnames['PASCAL VOC'].index(
+            'bicycle'):
         return "cyclist"  # YOLO Cyclist
     else:
         return "DontCare"
@@ -80,7 +85,7 @@ def calculate_class_scores_for_alfa(class_scores, detections, alfa_labels):
     cls_scs = detections[:, 7].numpy()
     for i in range(len(cls_scs)):
         # There are 20 ALFA class ids, the first element in the class score is for the "no object" case.
-        if alfa_labels[i]+1 < 21:
+        if alfa_labels[i] < len(dataset_classnames['PASCAL VOC']):
             class_scores[i, alfa_labels[i]+1] = cls_scs[i]
 
 
@@ -199,6 +204,26 @@ def convert_to_ai4adas_format(bboxes, label_ids, class_scores, ai4adas_objects,
     # print(ai4adas_alfa_full_detections)
 
 
+def dictionaries_to_ai4adas_format(classes_dict, scores_dict, bboxes_dict, ai4adas_objects, image_shape=(1, 1, 0)):
+    """Save dictionaries of bounding boxes et al. in AI4ADAS specific format.
+    """
+
+    h, w, _ = image_shape
+    if h == 0:
+        h = 1
+    if w == 0:
+        w = 1
+    scores = scores_dict['ALFA']
+    classes = classes_dict['ALFA']
+    bboxes = bboxes_dict['ALFA']
+    for i in range(classes.shape[0]):
+        cls_id = int(classes[i])
+        if cls_id >= 0:
+            score = scores[i][1:][cls_id]
+            ai4adas_objects.append({"obj_id": str(len(ai4adas_objects)), "class_name": str(dataset_classnames['PASCAL VOC'][classes[i]]),
+                                    "score": str(score), "xmin": str(bboxes[i, 0]/w), "ymin": str(bboxes[i, 1]/h), "xmax": str(bboxes[i, 2]/w), "ymax": str(bboxes[i, 3]/h)})
+
+
 def get_image_shape(detections, image_shapes):
 
     img_key = detections['img-name']
@@ -237,7 +262,7 @@ def from_ai4adas_to_alfa_format(ai4adas_alfa_detections, alfa_detections, image_
             alfa_class_scores = np.zeros((len(alfa_labels), 21))
             for i in range(len(class_scores)):
                 # There are 20 ALFA class ids, the first element in the class score is for the "no object" case.
-                if alfa_labels[i]+1 < 21:
+                if alfa_labels[i] < len(dataset_classnames['PASCAL VOC']):
                     alfa_class_scores[i, alfa_labels[i]+1] = class_scores[i]
             alfa_class_scores = alfa_class_scores.tolist()
 
@@ -246,13 +271,18 @@ def from_ai4adas_to_alfa_format(ai4adas_alfa_detections, alfa_detections, image_
         # print(alfa_detections)
 
 
-def save_ai4adas_object_as_json(results_dir, image_file_name, ai4adas_object):
-    """Save ALFA specific formatted object as json."""
+def transfer_ai4adas_object_as_json(results_dir, image_file_name, alfa_path, ai4adas_object):
+    """Save and transfer ALFA specific formatted object as json."""
 
     _, file_ext = os.path.splitext(image_file_name)
-    with open(f"{results_dir}/{image_file_name.replace(file_ext, '.json')}", 'w') as f:
+    image_file_json = image_file_name.replace(file_ext, '.json')
+    with open(f"{results_dir}/{image_file_json}", 'w') as f:
         json.dump(ai4adas_object, f, ensure_ascii=False)
 
+    if os.path.isdir(alfa_path):
+        shutil.copy(
+            f"{results_dir}/{image_file_json}", alfa_path)
+
 
 def transfer_detections_to_alfa_path(results_dir, alfa_json_name, alfa_path, ai4adas_alfa_detections):
     """Save and transfer ALFA specific formatted object (with all detections)."""
